@@ -5,17 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -50,10 +50,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -97,7 +97,12 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h,v 1.24 2005/02/05 06:57:24 fyodor Exp $ */
+/* $Id: NmapOps.h 6633 2007-12-22 06:32:03Z fyodor $ */
+
+#include "nmap.h"
+#include "global_structures.h"
+#include "output.h"
+#include <string>
 
 class NmapOps {
  public:
@@ -118,10 +123,13 @@ class NmapOps {
 // The time this obj. was instantiated   or last ReInit()ed.
   const struct timeval *getStartTime() { return &start_time; }
   // Number of milliseconds since getStartTime().  The current time is an
-  // optional argument to avoid an extre gettimeofday() call.
+  // optional argument to avoid an extra gettimeofday() call.
   int TimeSinceStartMS(struct timeval *now=NULL); 
   struct in_addr v4source();
   const struct in_addr *v4sourceip();
+
+
+
   bool TCPScan(); /* Returns true if at least one chosen scan type is TCP */
   bool UDPScan(); /* Returns true if at least one chosen scan type is UDP */
 
@@ -138,33 +146,69 @@ class NmapOps {
 		             user). */
   int isr00t;
   int debugging;
+
+#define PACKET_SEND_NOPREF 1
+#define PACKET_SEND_ETH_WEAK 2
+#define PACKET_SEND_ETH_STRONG 4
+#define PACKET_SEND_ETH 6
+#define PACKET_SEND_IP_WEAK 8
+#define PACKET_SEND_IP_STRONG 16
+#define PACKET_SEND_IP 24
+
+  /* How should we send raw IP packets?  Nmap can generally use either
+     ethernet or raw ip sockets.  Which is better depends on platform
+     and goals.  A _STRONG preference means that Nmap should use the
+     preferred method whenever it is possible (obviously it isn't
+     always possible -- sending ethernet frames won't work over a PPP
+     connection).  This is useful when the other type doesn't work at
+     all.  A _WEAK preference means that Nmap may use the other type
+     where it is substantially more efficient to do so. For example,
+     Nmap will still do an ARP ping scan of a local network even when
+     the pref is SEND_IP_WEAK */
+  int sendpref;
   bool packetTrace() { return (debugging >= 3)? true : pTrace;  }
   bool versionTrace() { return packetTrace()? true : vTrace;  }
+#ifndef NOLUA
+  bool scriptTrace() { return packetTrace()? true : scripttrace; }
+#endif
   // Note that packetTrace may turn on at high debug levels even if
   // setPacketTrace(false) has been called
   void setPacketTrace(bool pt) { pTrace = pt;  }
   void setVersionTrace(bool vt) { vTrace = vt;  }
+  bool openOnly() { return open_only; }
+  void setOpenOnly(bool oo) { open_only = oo; }
   int verbose;
   int randomize_hosts;
   int spoofsource; /* -S used */
+  int fastscan;
   char device[64];
   int interactivemode;
   int ping_group_sz;
   int generate_random_ips; /* -iR option */
-  FingerPrint **reference_FPs;
+  FingerPrintDB *reference_FPs; /* Used in the new OS scan system. */
   u16 magic_port;
   unsigned short magic_port_set; /* Was this set by user? */
   int num_ping_synprobes;
   /* The "synprobes" are also used when doing a connect() ping */
-  u16 ping_synprobes[MAX_PROBE_PORTS];
+  u16 *ping_synprobes;
   int num_ping_ackprobes;
-  u16 ping_ackprobes[MAX_PROBE_PORTS];
+  u16 *ping_ackprobes;
   int num_ping_udpprobes;
-  u16 ping_udpprobes[MAX_PROBE_PORTS];
+  u16 *ping_udpprobes;
+  int num_ping_protoprobes;
+  u16 *ping_protoprobes;
   /* Scan timing/politeness issues */
   int timing_level; // 0-5, corresponding to Paranoid, Sneaky, Polite, Normal, Aggressive, Insane
   int max_parallelism; // 0 means it has not been set
   int min_parallelism; // 0 means it has not been set
+  double topportlevel; // -1 means it has not been set
+
+  /* The maximum number of OS detection (gen2) tries we will make
+     without any matches before giving up on a host.  We may well give
+     up after fewer tries anyway, particularly if the target isn't
+     ideal for unknown fingerprint submissions */
+  int maxOSTries() { return max_os_tries; }
+  void setMaxOSTries(int mot);
 
   /* These functions retrieve and set the Round Trip Time timeouts, in
    milliseconds.  The set versions do extra processing to insure sane
@@ -176,6 +220,8 @@ class NmapOps {
   void setMaxRttTimeout(int rtt);
   void setMinRttTimeout(int rtt);
   void setInitialRttTimeout(int rtt);
+  void setMaxRetransmissions(int max_retransmit);
+  int getMaxRetransmissions() { return max_retransmissions; }
 
   /* Similar functions for Host group size */
   int minHostGroupSz() { return min_host_group_sz; }
@@ -197,6 +243,11 @@ class NmapOps {
      should be skipped */
   char *XSLStyleSheet() { return xsl_stylesheet; }
 
+  /* Sets the spoofed MAC address */
+  void setSpoofMACAddress(u8 *mac_data);
+  /* Gets the spoofed MAC address, but returns NULL if it hasn't been set */
+  const u8 *spoofMACAddress() { return spoof_mac_set? spoof_mac : NULL; }
+
   int max_ips_to_scan; // Used for Random input (-iR) to specify how 
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data_length op */
@@ -204,18 +255,27 @@ class NmapOps {
   unsigned long host_timeout;
   /* Delay between probes, in milliseconds */
   unsigned int scan_delay;
+  bool open_only;
 
   int scanflags; /* if not -1, this value should dictate the TCP flags
 		    for the core portscaning routine (eg to change a
 		    FIN scan into a PSH scan.  Sort of a hack, but can
 		    be very useful sometimes. */
 
+  int defeat_rst_ratelimit; /* Solaris 9 rate-limits RSTs so scanning is very
+            slow against it. If we don't distinguish between closed and filtered ports,
+            we can get the list of open ports very fast */
+
   struct in_addr resume_ip; /* The last IP in the log file if user 
 			       requested --restore .  Otherwise 
 			       restore_ip.s_addr == 0.  Also 
 			       target_struct_get will eventually set it 
 			       to 0. */
 
+  // Version Detection Options
+  int override_excludeports;
+  int version_intensity;
+
   struct in_addr decoys[MAX_DECOYS];
   int osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
   int osscan_guess;   /* Be more aggressive in guessing OS type */
@@ -242,16 +302,57 @@ class NmapOps {
   int windowscan;
   int xmasscan;
   int noresolve;
-  int force; /* force nmap to continue on even when the outcome seems somewhat certain */
   int append_output; /* Append to any output files rather than overwrite */
-  FILE *logfd[LOG_TYPES];
+  FILE *logfd[LOG_NUM_FILES];
   FILE *nmap_stdout; /* Nmap standard output */
   int ttl; // Time to live
+  int badsum;
   char *datadir;
+  /* A map from abstract data file names like "nmap-services" and "nmap-os-db"
+     to paths which have been requested by the user. nmap_fetchfile will return
+     the file names defined in this map instead of searching for a matching
+     file. */
+  std::map<std::string, std::string> requested_data_files;
+  /* A map from data file names to the paths at which they were actually found.
+     Only files that were actually read should be in this map. */
+  std::map<std::string, std::string> loaded_data_files;
+  bool mass_dns;
+  int resolve_all;
+  char *dns_servers;
+  bool log_errors;
+  bool traceroute;
+  bool reason;
+
+#ifndef NOLUA
+  int script;
+  char *scriptargs;
+  int scriptversion;
+  int scripttrace;
+  int scriptupdatedb;
+  void chooseScripts(char* argument);
+  std::vector<std::string> chosenScripts;
+#endif
+
+  /* ip options used in build_*_raw() */
+  u8 *ipoptions;
+  int ipoptionslen;
+  int ipopt_firsthop;	// offset in ipoptions where is first hop for source/strict routing
+  int ipopt_lasthop;	// offset in ipoptions where is space for targets ip for source/strict routing
+
+  // Statistics Options set in nmap.cc
+  int numhosts_scanned;
+  int numhosts_up;
+  int numhosts_scanning;
+  stype current_scantype;
+  bool noninteractive;
+
+  bool release_memory;	/* suggest to release memory before quitting. used to find memory leaks. */
  private:
+  int max_os_tries;
   int max_rtt_timeout;
   int min_rtt_timeout;
   int initial_rtt_timeout;
+  int max_retransmissions;
   unsigned int max_tcp_scan_delay;
   unsigned int max_udp_scan_delay;
   unsigned int min_host_group_sz;
@@ -264,5 +365,7 @@ class NmapOps {
   bool pTrace; // Whether packet tracing has been enabled
   bool vTrace; // Whether version tracing has been enabled
   char *xsl_stylesheet;
+  u8 spoof_mac[6];
+  bool spoof_mac_set;
 };
   
