@@ -6,17 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -39,7 +39,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -51,10 +51,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -103,11 +103,110 @@
 #include <winclude.h>
 #include <sys/timeb.h>
 
-#include "nmap_error.h"
+
+#include "..\nmap.h"
+#include "..\tcpip.h"
+#include "winfix.h"
+#include "..\NmapOps.h"
+#include "..\nmap_error.h"
+
+#ifdef _MSC_VER
+# include <delayimp.h>
+#endif
+
+#ifdef _MSC_VER
+#define DLI_ERROR VcppException(ERROR_SEVERITY_ERROR, ERROR_MOD_NOT_FOUND)
+#endif
+
+extern NmapOps o;
+
+int pcap_avail = 0;
+
+/*   internal functions   */
+static void win_cleanup(void);
+static char pcaplist[4096];
+
+/* The code that has no preconditions to being called, so it can be
+   executed before even Nmap options parsing (so o.debugging and the
+   like don't need to be used.  Its main function is to do
+   WSAStartup() as some of the option parsing code does DNS
+   resolution */
+void win_pre_init() {
+	WORD werd;
+	WSADATA data;
+
+	werd = MAKEWORD( 2, 2 );
+	if( (WSAStartup(werd, &data)) !=0 )
+		fatal("failed to start winsock.\n");
+}
+
+/* Requires that win_pre_init() has already been called, also that
+   options processing has been done so that o.debugging is
+   available */
+void win_init()
+{
+	//   variables
+	DWORD cb = 0;
+	DWORD nRes;
+	OSVERSIONINFOEX ver;
+	PMIB_IPADDRTABLE pIp = 0;
+	int i;
+	int numipsleft;
+
+
+	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+	if(!GetVersionEx((LPOSVERSIONINFO)&ver))
+	{
+		ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+		if(!GetVersionEx((LPOSVERSIONINFO)&ver))
+			fatal("GetVersionEx failed\n");
+
+		ver.wServicePackMajor = 0;
+		ver.wServicePackMinor = 0;
+	}
+
+
+	//   Try to initialize winpcap
+#ifdef _MSC_VER
+	__try
+#endif
+	{
+		ULONG len = sizeof(pcaplist);
+
+		pcap_avail = 1;
+		if(o.debugging > 2) printf("***WinIP***  trying to initialize WinPcap\n");
+		PacketGetAdapterNames(pcaplist, &len);
+
+#ifdef _MSC_VER
+		if(FAILED(__HrLoadAllImportsForDll("wpcap.dll")))
+		{
+			error("WARNING: your winpcap is too old to use.  Nmap may not function.\n");
+			pcap_avail = 0;
+		}
+#endif
+		if(o.debugging)
+			printf("Winpcap present, dynamic linked to: %s\n", pcap_lib_version());
+	}
+#ifdef _MSC_VER
+	__except (1) {
+			error("WARNING: Could not import all necessary WinPcap functions.  You may need to upgrade to version 3.1 or higher from http://www.winpcap.org.  Resorting to connect() mode -- Nmap may not function completely");
+		pcap_avail=0;
+		}
+#endif
+
+	if (!pcap_avail)
+		o.isr00t = 0;
+	atexit(win_cleanup);
+}
+
+
+static void win_cleanup(void)
+{
+  WSACleanup();
+}
 
 int my_close(int sd)
 {
-	if(sd == 501) return 0;
 	return closesocket(sd);
 }
 
