@@ -9,17 +9,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -42,7 +42,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -54,10 +54,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h 3438 2006-06-10 21:23:30Z fyodor $ */
+/* $Id: nbase.h 6636 2007-12-22 06:32:28Z fyodor $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -122,7 +122,7 @@
  *   will generally be one of the following: LINUX, FREEBSD, NETBSD,
  *   OPENBSD, SOLARIS, SUNOS, BSDI, IRIX, NETBSD
  *
- * * Insures that GNU getopt_* functions exist (such as getopt_long_only
+ * * Insures that getopt_* functions exist (such as getopt_long_only)
  *
  * * Various string functions such as Strncpy() and strcasestr() see protos 
  *   for more info.
@@ -270,17 +270,12 @@ extern "C" int snprintf (char *, size_t, const char *, ...);
 extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #endif
 
-/* GNU getopt replacements ... Anyone have a BSD licensed version of these? */
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #else
-/* The next half-dozen lines are from gcc-2.95 ... -Fyodor */
-/* Include getopt.h for the sake of getopt_long.
-   We don't need the declaration of getopt, and it could conflict
-   with something from a system header file, so effectively nullify that.  */
-#define getopt getopt_loser
+#ifndef HAVE_GETOPT_LONG_ONLY
 #include "getopt.h"
-#undef getopt
+#endif
 #endif /* HAVE_GETOPT_H */
 
 /* More Windows-specific stuff */
@@ -293,6 +288,18 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define S_ISDIR(m)      (((m) & _S_IFMT) == _S_IFDIR)
 #endif
 
+/* Windows doesn't have the access() defines */
+#ifndef F_OK
+#define F_OK 00
+#endif
+#ifndef W_OK
+#define W_OK 02
+#endif
+#ifndef R_OK
+#define R_OK 04
+#endif
+
+#define access _access
 #define stat _stat /* wtf was ms thinking? */
 #define execve _execve
 #define getpid _getpid
@@ -311,7 +318,7 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
-
+#define execv _execv
 
 #endif /* WIN32 */
 
@@ -373,6 +380,10 @@ unsigned int sleep(unsigned int seconds);
    it must truncate */
 int Strncpy(char *dest, const char *src, size_t n);
 
+int Vsnprintf(char *, size_t, const char *, va_list)
+     __attribute__ ((format (printf, 3, 0)));
+int Snprintf(char *, size_t, const char *, ...)
+     __attribute__ ((format (printf, 3, 4)));
 
 /* Trivial function that returns nonzero if all characters in str of
    length strlength are printable (as defined by isprint()) */
@@ -381,6 +392,10 @@ int stringisprintable(const char *str, int strlength);
 /* Convert non-printable characters to replchar in the string */
 void replacenonprintable(char *str, int strlength, char replchar);
 
+/* Portable, incompatible replacements for dirname and basename. */
+char *path_get_dirname(const char *path);
+char *path_get_basename(const char *path);
+
 /* A few simple wrappers for the most common memory allocation routines which will exit() if the
 	allocation fails, so you don't always have to check -- see nbase_memalloc.c */
 void *safe_malloc(size_t size);
@@ -398,6 +413,8 @@ u32 get_random_u32();
 u16 get_random_u16();
 u8 get_random_u8();
 
+/* CRC16 Cyclic Redundancy Check  */
+unsigned long crc16(unsigned char *buf, int len);
 
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
