@@ -4,17 +4,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -37,7 +37,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -49,10 +49,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -142,6 +142,9 @@
 // doug at hcsw.org
 // http://www.hcsw.org
 
+/*
+ * DNS Caching and ageing added by Eddie Bell ejlbell@gmail.com 2007
+ */
 
 // TODO:
 //
@@ -158,6 +161,7 @@
 #include <limits.h>
 #include <list>
 #include <vector>
+#include <algorithm>
 
 #include "nmap.h"
 #include "NmapOps.h"
@@ -165,6 +169,8 @@
 #include "nsock.h"
 #include "utils.h"
 #include "nmap_tty.h"
+#include "timing.h"
+#include "Target.h"
 
 extern NmapOps o;
 
@@ -267,6 +273,7 @@ struct request_s {
 struct host_elem_s {
   char *name;
   u32 addr;
+  u8 cache_hits;
 };
 
 
@@ -291,14 +298,14 @@ static ScanProgressMeter *SPM;
 
 //------------------- Prototypes and macros ---------------------
 
-void put_dns_packet_on_wire(request *req);
+static void put_dns_packet_on_wire(request *req);
+static char *lookup_etchosts(u32 ip);
+static void addto_etchosts(u32 ip, const char *hname);
 
 #define ACTION_FINISHED 0
 #define ACTION_CNAME_LIST 1
 #define ACTION_TIMEOUT 2
 
-
-
 //------------------- Misc code --------------------- 
 
 static void output_summary() {
@@ -370,7 +377,8 @@ static void do_possible_writes() {
       }
 
       if (tpreq) {
-        if (o.debugging >= TRACE_DEBUG_LEVEL) log_write(LOG_STDOUT, "mass_rdns: TRANSMITTING for <%s> (server <%s>)\n", tpreq->targ->targetipstr() , tpserv->hostname);
+        if (o.debugging >= TRACE_DEBUG_LEVEL)
+	   log_write(LOG_STDOUT, "mass_rdns: TRANSMITTING for <%s> (server <%s>)\n", tpreq->targ->targetipstr() , tpserv->hostname);
         stat_trans++;
         put_dns_packet_on_wire(tpreq);
       }
@@ -391,14 +399,13 @@ static void write_evt_handler(nsock_pool nsp, nsock_event evt, void *req_v) {
 // Takes a DNS request structure and actually puts it on the wire
 // (calls nsock_write()). Does various other tasks like recording
 // the time for the timeout.
-void put_dns_packet_on_wire(request *req) {
+static void put_dns_packet_on_wire(request *req) {
   char packet[512];
   int plen=0;
   u32 ip;
   struct timeval now, timeout;
 
   ip = (u32) ntohl(req->targ->v4host().s_addr);
-
   packet[0] = (req->id >> 8) & 0xFF;
   packet[1] = req->id & 0xFF;
   plen += 2;
@@ -527,13 +534,18 @@ static int process_result(u32 ia, char *result, int action, u16 id) {
 
       if (id == tpreq->id) {
 
-        if (ia != 0 && (u32) (tpreq->targ->v4host().s_addr) != ia) continue;
+        if (ia != 0 && tpreq->targ->v4host().s_addr != ia)
+          continue;
 
         if (action == ACTION_CNAME_LIST || action == ACTION_FINISHED) {
         tpserv->capacity += CAPACITY_UP_STEP;
         check_capacities(tpserv);
 
-        if (result) tpreq->targ->setHostName(result);
+        if (result) {
+          tpreq->targ->setHostName(result);
+          addto_etchosts(tpreq->targ->v4hostip()->s_addr, result);
+        }
+
         tpserv->in_process.remove(tpreq);
         tpserv->reqs_on_wire--;
 
@@ -569,9 +581,9 @@ static u32 parse_inaddr_arpa(unsigned char *buf, int maxlen) {
   u32 ip=0;
   int i, j;
 
-  for (i=0; i<=3; i++) {
-    if (maxlen <= 0) return 0;
+  if (maxlen <= 0) return 0;
 
+  for (i=0; i<=3; i++) {
     if (buf[0] < 1 || buf[0] > 3) return 0;
 
     maxlen -= buf[0] + 1;
@@ -593,7 +605,7 @@ static u32 parse_inaddr_arpa(unsigned char *buf, int maxlen) {
 // Turns a DNS packet encoded name (see the RFC) and turns it into
 // a normal decimal separated hostname.
 // ASSUMES NAME LENGTH/VALIDITY HAS ALREADY BEEN VERIFIED
-int encoded_name_to_normal(unsigned char *buf, char *output, int outputsize){
+static int encoded_name_to_normal(unsigned char *buf, char *output, int outputsize){
   while (buf[0]) {
     if (buf[0] >= outputsize-1) return -1;
     memcpy(output, buf+1, buf[0]);
@@ -657,9 +669,9 @@ static void read_evt_handler(nsock_pool nsp, nsock_event evt, void *nothing) {
 
   if (nse_type(evt) != NSE_TYPE_READ || nse_status(evt) != NSE_STATUS_SUCCESS) {
     if (o.debugging)
-      log_write(LOG_STDOUT, "mass_dns: warning: got a %s:%s in read_evt_handler()\n",
+      log_write(LOG_STDOUT, "mass_dns: warning: got a %s:%s in %s()\n",
         nse_type2str(nse_type(evt)),
-        nse_status2str(nse_status(evt)));
+        nse_status2str(nse_status(evt)), __func__);
     return;
   }
 
@@ -674,9 +686,10 @@ static void read_evt_handler(nsock_pool nsp, nsock_event evt, void *nothing) {
   // 0xFA == 11111010 (we're not concerned with AA or RD bits)
   if ((buf[2] & 0xFA) != 0x80) return;
 
-  // Check that Recursion is available, the zero field is all zeros
-  // and there is no error condition:
-  if (buf[3] != 0x80) {
+  // Check that the zero field is all zeros and there is no error condition.
+  // We don't care if recursion is available or not since we might be querying
+  // an authoritative DNS server.
+  if (buf[3] != 0x80 && buf[3] != 0) {
     if ((buf[3] & 0xF) == 2) errcode = 2;
     else if ((buf[3] & 0xF) == 3) errcode = 3;
     else return;
@@ -816,9 +829,24 @@ static void add_dns_server(char *ipaddrs) {
 
 }
 
+void free_dns_servers() {
+  std::list<dns_server *>::iterator servI;
+  dns_server *tpserv;
+
+  for(servI = servs.begin(); servI != servs.end();servI++){
+    tpserv = *servI;
+    if(tpserv){
+      if(tpserv->hostname)
+        free(tpserv->hostname);
+      delete tpserv;
+    }
+  }
+  servs.clear();
+}
+
 
 // Creates a new nsi for each DNS server
-void connect_dns_servers() {
+static void connect_dns_servers() {
   std::list<dns_server *>::iterator serverI;
   dns_server *s;
 
@@ -846,7 +874,7 @@ void win32_read_registry(char *controlset) {
   char buf[2048], keyname[2048], *p;
   DWORD sz, i;
 
-  snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters", controlset);
+  Snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters", controlset);
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keybasebuf,
                     0, KEY_READ, &hKey) != ERROR_SUCCESS) {
     if (firstrun) error("mass_dns: warning: Error opening registry to read DNS servers. Try using --system-dns or specify valid servers with --dns-servers");
@@ -863,14 +891,14 @@ void win32_read_registry(char *controlset) {
 
   RegCloseKey(hKey);
 
-  snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces", controlset);
+  Snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces", controlset);
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keybasebuf,
                     0, KEY_ENUMERATE_SUB_KEYS, &hKey) == ERROR_SUCCESS) {
 
     sz = sizeof(buf);
     for (i=0; RegEnumKeyEx(hKey, i, buf, &sz, NULL, NULL, NULL, NULL) != ERROR_NO_MORE_ITEMS; i++) {
 
-      snprintf(keyname, sizeof(keyname), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces\\%s", controlset, buf);
+      Snprintf(keyname, sizeof(keyname), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces\\%s", controlset, buf);
 
       if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname,
                         0, KEY_READ, &hKey2) == ERROR_SUCCESS) {
@@ -961,6 +989,7 @@ static void parse_etchosts(char *fname) {
         he = new host_elem;
         he->name = strdup(hname);
         he->addr = (u32) ia.s_addr;
+        he->cache_hits = 0;
         etchosts[he->addr % HASH_TABLE_SIZE].push_front(he);
       }
     }
@@ -969,19 +998,83 @@ static void parse_etchosts(char *fname) {
   fclose(fp);
 }
 
+void free_etchosts() {
+  host_elem *he;
+  std::list<host_elem *>::iterator hi;
+  int i;
 
+  for(i=0; i < HASH_TABLE_SIZE; i++){
+    for(hi = etchosts[i].begin(); hi != etchosts[i].end(); hi++) {
+      he = *hi;
+      if(he) {
+        free(he->name);
+        delete he;
+      }
+    }
+    etchosts[i].clear();
+  }
+}
+
+/* Executed when the DNS cache is full, ages entries
+ * and removes any with a cache hit of 0 (the least used) */
+bool remove_and_age(host_elem *host) {
+  if(host->cache_hits) {
+     host->cache_hits /=2;
+     return false;
+  } else
+     return true;
+}
+
+/* Add to the dns cache. If there is no space we age and 
+ * remove the least frequently used entries until there 
+ * is space */
+static void addto_etchosts(u32 ip, const char *hname) {
+  static u16 total_size = 0;
+  std::list<host_elem*>::iterator it;
+  host_elem *he;
+  int i;
+
+  if(lookup_etchosts(ip) != NULL) 
+    return;
+
+  while(total_size >= HASH_TABLE_SIZE) {
+    for(i = 0; i < HASH_TABLE_SIZE; i++) {
+      while((it = find_if(etchosts[i].begin(), etchosts[i].end(), remove_and_age)) != etchosts[i].end()) {
+        etchosts[i].erase(it);
+        total_size--;
+      }
+    }
+  }
+  he = new host_elem;
+  he->name = strdup(hname);
+  he->addr = ip;
+  he->cache_hits = 0;
+  etchosts[ip % HASH_TABLE_SIZE].push_back(he);
+  total_size++;
+}
+
+/* Search for a hostname in the cache and increment
+ * its cache hit counter if found */
 static char *lookup_etchosts(u32 ip) {
   std::list<host_elem *>::iterator hostI;
   host_elem *tpelem;
 
   for(hostI = etchosts[ip % HASH_TABLE_SIZE].begin(); hostI != etchosts[ip % HASH_TABLE_SIZE].end(); hostI++) {
     tpelem = *hostI;
-    if (tpelem->addr == ip) return tpelem->name;
+    if (tpelem->addr == ip) {
+      if(tpelem->cache_hits < UCHAR_MAX)
+        tpelem->cache_hits++;
+      return tpelem->name;
+    }
   }
-
   return NULL;
 }
 
+/* External interface to dns cache */
+const char *lookup_cached_host(u32 ip) {
+  const char *tmp = lookup_etchosts(ip);
+  return tmp==NULL?"":tmp;
+}
 
 static void etchosts_init(void) {
   static int initialized = 0;
@@ -1000,11 +1093,11 @@ static void etchosts_init(void) {
   has_backslash = (windows_dir[strlen(windows_dir)-1] == '\\');
 
   // Windows 95/98/Me:
-  snprintf(tpbuf, sizeof(tpbuf), "%s%shosts", windows_dir, has_backslash ? "" : "\\");
+  Snprintf(tpbuf, sizeof(tpbuf), "%s%shosts", windows_dir, has_backslash ? "" : "\\");
   parse_etchosts(tpbuf);
 
   // Windows NT/2000/XP/2K3:
-  snprintf(tpbuf, sizeof(tpbuf), "%s%ssystem32\\drivers\\etc\\hosts", windows_dir, has_backslash ? "" : "\\");
+  Snprintf(tpbuf, sizeof(tpbuf), "%s%ssystem32\\drivers\\etc\\hosts", windows_dir, has_backslash ? "" : "\\");
   parse_etchosts(tpbuf);
 
 #else
@@ -1026,6 +1119,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   char *tpname;
   int i;
   bool lasttrace = false;
+  char spmobuf[1024];
 
   if (o.mass_dns == false) {
     Target *currenths;
@@ -1039,8 +1133,9 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
       if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) stat_actual++;
     }
 
-    SPM = new ScanProgressMeter("System DNS resolution");
-    
+    Snprintf(spmobuf, sizeof(spmobuf), "System DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
+    SPM = new ScanProgressMeter(spmobuf);
+
     for(i=0, hostI = targets; hostI < targets+num_targets; hostI++, i++) {
       currenths = *hostI;
 	
@@ -1058,6 +1153,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
       }
     }
 
+    SPM->endTask(NULL, NULL);
     delete SPM;
 
     return;
@@ -1083,7 +1179,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   for(hostI = targets; hostI < targets+num_targets; hostI++) {
     if (!((*hostI)->flags & HOST_UP) && !o.resolve_all) continue;
 
-    // See if it's in /etc/hosts
+    // See if it's in /etc/hosts or cached
     tpname = lookup_etchosts((u32) (*hostI)->v4hostip()->s_addr);
     if (tpname) {
       (*hostI)->setHostName(tpname);
@@ -1107,7 +1203,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   // And finally, do it!
 
   if ((dnspool = nsp_new(NULL)) == NULL)
-    fatal("Unable to create nsock pool in nmap_mass_rdns_core()");
+    fatal("Unable to create nsock pool in %s()", __func__);
 
   if ((lasttrace = o.packetTrace()))
     nsp_settrace(dnspool, 5, o.getStartTime());
@@ -1118,7 +1214,8 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
 
   read_timeout_index = MIN(sizeof(read_timeouts)/sizeof(read_timeouts[0]), servs.size()) - 1;
 
-  SPM = new ScanProgressMeter("System DNS resolution");
+  Snprintf(spmobuf, sizeof(spmobuf), "Parallel DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
+  SPM = new ScanProgressMeter(spmobuf);
 
   while (total_reqs > 0) {
     timeout = deal_with_timedout_reads();
@@ -1137,6 +1234,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     nsock_loop(dnspool, timeout);
   }
 
+  SPM->endTask(NULL, NULL);
   delete SPM;
 
   close_dns_servers();
@@ -1146,34 +1244,38 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   if (cname_reqs.size() && o.debugging)
     log_write(LOG_STDOUT, "Performing system-dns for %d domain names that use CNAMEs\n", (int) cname_reqs.size());
 
-  SPM = new ScanProgressMeter("System CNAME DNS resolution");
+  if (cname_reqs.size()) {
+    Snprintf(spmobuf, sizeof(spmobuf), "System CNAME DNS resolution of %u host%s.", (unsigned) cname_reqs.size(), cname_reqs.size()-1 ? "s" : "");
+    SPM = new ScanProgressMeter(spmobuf);
 
-  for(i=0, reqI = cname_reqs.begin(); reqI != cname_reqs.end(); reqI++, i++) {
-    struct sockaddr_storage ss;
-    size_t sslen;
-    char hostname[MAXHOSTNAMELEN + 1] = "";
+    for(i=0, reqI = cname_reqs.begin(); reqI != cname_reqs.end(); reqI++, i++) {
+      struct sockaddr_storage ss;
+      size_t sslen;
+      char hostname[MAXHOSTNAMELEN + 1] = "";
 
-    if (keyWasPressed())
-      SPM->printStats((double) i / cname_reqs.size(), NULL);
+      if (keyWasPressed())
+        SPM->printStats((double) i / cname_reqs.size(), NULL);
 
-    tpreq = *reqI;
+      tpreq = *reqI;
 
-    if (tpreq->targ->TargetSockAddr(&ss, &sslen) != 0)
-      fatal("Failed to get target socket address.");
+      if (tpreq->targ->TargetSockAddr(&ss, &sslen) != 0)
+        fatal("Failed to get target socket address.");
+
+      if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
+                      sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
+        stat_ok++;
+        stat_cname++;
+        tpreq->targ->setHostName(hostname);
+      }
+
+      delete tpreq;
 
-    if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
-                    sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
-      stat_ok++;
-      stat_cname++;
-      tpreq->targ->setHostName(hostname);
     }
 
-    delete tpreq;
-
+    SPM->endTask(NULL, NULL);
+    delete SPM;
   }
 
-  delete SPM;
-
   cname_reqs.clear();
 
 }
@@ -1195,7 +1297,7 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
   gettimeofday(&now, NULL);
 
   if (stat_actual > 0) {
-    if (o.debugging) {
+    if (o.debugging || o.verbose >= 3) {
       if (o.mass_dns) {
 	// #:  Number of DNS servers used
 	// OK: Number of fully reverse resolved queries
@@ -1211,8 +1313,6 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
 		  stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0,
 		  stat_ok, stat_actual - stat_ok);
       }
-    } else if (o.verbose) {
-      log_write(LOG_STDOUT, "DNS resolution of %d IPs took %.2fs.\n", stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0);
     }
   }
 
