@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -96,7 +96,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_error.cc 3869 2006-08-25 01:47:49Z fyodor $ */
+/* $Id: nmap_error.cc 6633 2007-12-22 06:32:03Z fyodor $ */
 
 #include "nmap_error.h"
 #include "output.h"
@@ -167,7 +167,7 @@ void pfatal(const char *err, ...) {
   log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
 	    strerror(errno), errno);
 #endif /* WIN32 perror() compatability switch */
-  if (o.log_errors) log_flush(LOG_STDOUT);
+  if (o.log_errors) log_flush(LOG_NORMAL);
   fflush(stderr);
   exit(1);
 }
@@ -204,7 +204,7 @@ void gh_perror(const char *err, ...) {
   log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
 	    strerror(errno), errno);
 #endif /* WIN32 perror() compatability switch */
-  if (o.log_errors) log_flush(LOG_STDOUT);
+  if (o.log_errors) log_flush(LOG_NORMAL);
   fflush(stderr);
   return;
 }
