@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -54,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: netutils.h,v 1.5 2004/08/29 09:12:05 fyodor Exp $ */
+/* $Id: netutils.h 6635 2007-12-22 06:32:18Z fyodor $ */
 
 #ifndef NETUTILS_H
 #define NETUTILS_H
@@ -66,6 +66,8 @@
 
 #ifdef WIN32
 #include "nbase_winconfig.h"
+/* nbase_winunix.h somehow reason.h to get included */
+#include "nbase_winunix.h"
 #endif
 
 /* Take a file descriptor and put it in non-blocking mode */
