@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -55,7 +55,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_internal.h 2443 2004-10-12 09:34:12Z fyodor $ */
+/* $Id: nsock_internal.h 6635 2007-12-22 06:32:18Z fyodor $ */
 
 #ifndef NSOCK_INTERNAL_H
 #define NSOCK_INTERNAL_H
@@ -136,6 +136,9 @@ struct event_lists {
   gh_list read_events;
   gh_list write_events;
   gh_list timer_events;
+  #if HAVE_PCAP
+  gh_list pcap_read_events;
+  #endif 
   gh_list free_events; /* When an event is deleted, we stick it here for
 			  later reuse */
   struct timeval next_ev; /* The soonest time that either a timer event goes
@@ -226,6 +229,8 @@ struct msiod {
   unsigned long id; /* Every iod has an id which is always unique for the
 		       same nspool (unless you create billions of them) */
   void *userdata;
+  
+  void *pcap;	   /* Pointer to mspcap struct (used only if pcap support is included) */
 };
 
 
@@ -321,6 +326,11 @@ void handle_write_result(mspool *ms, msevent *nse,
 void handle_timer_result(mspool *ms, msevent *nse, 
 			 enum nse_status status);
 
+#if HAVE_PCAP
+void handle_pcap_read_result(mspool *ms, msevent *nse, 
+			       enum nse_status status);
+#endif
+
 void nsock_trace(mspool *ms, char *fmt, ...)
      __attribute__ ((format (printf, 2, 3)));
 
