@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -56,13 +56,17 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_iod.c,v 1.20 2004/08/29 09:12:05 fyodor Exp $ */
+/* $Id: nsock_iod.c 6635 2007-12-22 06:32:18Z fyodor $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
 #include "gh_list.h"
 #include "netutils.h"
 
+#if HAVE_PCAP
+#include "nsock_pcap.h"
+#endif
+
 #include <string.h>
 
 /* nsock_iod is like a "file descriptor" for the nsock library.  You
@@ -95,7 +99,10 @@ nsock_iod nsi_new2(nsock_pool nsockp, int sd, void *userdata) {
     nsi->state = NSIOD_STATE_INITIAL;
   } else {
     nsi->sd = dup(sd);
-    if (nsi->sd == -1) return NULL;
+    if (nsi->sd == -1) {
+      free(nsi);
+      return NULL;
+    }
     nsock_unblock_socket(nsi->sd);
     nsi->state = NSIOD_STATE_UNKNOWN;
   }
@@ -159,8 +166,8 @@ void nsi_delete(nsock_iod nsockiod, int pending_response) {
 	currev = (msevent *) GH_LIST_ELEM_DATA(currev_elem);
 	next_elem = GH_LIST_ELEM_NEXT(currev_elem);
 	if (currev->iod == nsi) {
-	  // OK - we found an event pending on this IOD.  Kill it.
-	  // printf("Found an outstanding event (out of %d), removing\n", nsi->events_pending);
+	  /* OK - we found an event pending on this IOD.  Kill it. */
+	  /* printf("Found an outstanding event (out of %d), removing\n", nsi->events_pending); */
 	  msevent_cancel(nsi->nsp, currev, elist_ar[elist], currev_elem, pending_response == NSOCK_PENDING_NOTIFY);
 	}
 	if (nsi->events_pending == 0)
@@ -204,6 +211,26 @@ void nsi_delete(nsock_iod nsockiod, int pending_response) {
   nsi->state = NSIOD_STATE_DELETED;
   nsi->userdata = NULL;
 
+#if HAVE_PCAP
+  if(nsi->pcap){
+    mspcap *mp = (mspcap *) nsi->pcap;
+    if(mp->pt){
+      pcap_close(mp->pt);
+      mp->pt=NULL;
+    }
+    if(mp->pcap_desc){
+      // Should I close pcap_desc or pcap_close does this for me?
+      mp->pcap_desc = -1;
+    }
+    if(mp->pcap_device){
+    	free(mp->pcap_device);
+    	mp->pcap_device = NULL;
+    }
+    free(mp);
+    nsi->pcap = NULL;
+  }
+#endif
+
   gh_list_remove_elem(&nsi->nsp->active_iods, nsi->entry_in_nsp_active_iods);
   gh_list_prepend(&nsi->nsp->free_iods, nsi);
 
