@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2006 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -54,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_write.c 3870 2006-08-25 01:47:53Z fyodor $ */
+/* $Id: nsock_write.c 6635 2007-12-22 06:32:18Z fyodor $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
@@ -122,13 +122,13 @@ nsock_event_id nsock_printf(nsock_pool ms_pool, nsock_iod ms_iod,
 		    userdata);
   assert(nse);
 
-  res = vsnprintf(buf, sizeof(buf), format, ap);
+  res = Vsnprintf(buf, sizeof(buf), format, ap);
   va_end(ap);
 
   if (res != -1) {
     if (res > sizeof(buf)) {
       buf2 = (char * ) safe_malloc(res + 16);
-      res2 = vsnprintf(buf2, sizeof(buf), format, ap);
+      res2 = Vsnprintf(buf2, sizeof(buf), format, ap);
       if (res2 == -1 || res2 > res) {
 	free(buf2);
 	buf2 = NULL;
