@@ -5,17 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -50,10 +50,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -97,28 +97,72 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: utils.cc,v 1.22 2004/08/29 09:12:04 fyodor Exp $ */
+/* $Id: utils.cc 6633 2007-12-22 06:32:03Z fyodor $ */
 
+#include "nmap.h"
 #include "utils.h"
 #include "NmapOps.h"
 
 extern NmapOps o;
 
+
+
+/* Test a wildcard mask against a test string. Wildcard mask
+ * can include '*' and '?' which work the same as they do
+ * in /bin/sh (except it's case insensitive)
+ * Return val of 1 means it DID match. 0 means it DIDN'T
+ * - Doug Hoyte, 2005
+ */
+
+int wildtest(char *wild, char *test) {
+
+  int i;
+
+  while(*wild != '\0'  ||  *test != '\0') {
+    if (*wild == '*') {
+
+      /* --- Deal with multiple asterisks. --- */
+      while (wild[1] == '*') wild++;
+
+      /* --- Deal with terminating asterisks. --- */
+      if (wild[1] == '\0') return 1;
+
+      for(i=0; test[i]!='\0'; i++)
+        if ((tolower((int)wild[1]) == tolower((int)test[i]) || wild[1] == '?')
+            &&  wildtest(wild+1, test+i) == 1) return 1;
+
+      return 0;
+    }
+
+    /* --- '?' can't match '\0'. --- */
+    if (*wild == '?' && *test == '\0') return 0;
+
+    if (*wild != '?' && tolower((int)*wild) != tolower((int)*test)) return 0;
+    wild++; test++;
+  }
+
+  if (tolower((int)*wild) == tolower((int)*test)) return 1;
+  return 0;
+
+}
+
+
+
 /* Hex dump */
 void hdump(unsigned char *packet, unsigned int len) {
 unsigned int i=0, j=0;
 
-printf("Here it is:\n");
+log_write(LOG_PLAIN, "Here it is:\n");
 
 for(i=0; i < len; i++){
   j = (unsigned) (packet[i]);
-  printf("%-2X ", j);
+  log_write(LOG_PLAIN, "%-2X ", j);
   if (!((i+1)%16))
-    printf("\n");
+    log_write(LOG_PLAIN, "\n");
   else if (!((i+1)%4))
-    printf("  ");
+    log_write(LOG_PLAIN, "  ");
 }
-printf("\n");
+log_write(LOG_PLAIN, "\n");
 }
 
 /* A better version of hdump, from Lamont Granquist.  Modified slightly
@@ -136,7 +180,7 @@ void lamont_hdump(char *cp, unsigned int length) {
   int nshorts, nshorts2;
   int padding;
   
-  printf("\n\t");
+  log_write(LOG_PLAIN, "\n\t");
   padding = 0;
   sp = (u_short *)bp;
   ap = (u_char *)bp;
@@ -146,44 +190,44 @@ void lamont_hdump(char *cp, unsigned int length) {
   j = 0;
   while(1) {
     while (--nshorts >= 0) {
-      printf(" %04x", ntohs(*sp));
+      log_write(LOG_PLAIN, " %04x", ntohs(*sp));
       sp++;
       if ((++i % 8) == 0)
         break;
     }
     if (nshorts < 0) {
       if ((length & 1) && (((i-1) % 8) != 0)) {
-        printf(" %02x  ", *(u_char *)sp);
+        log_write(LOG_PLAIN, " %02x  ", *(u_char *)sp);
         padding++;
       }
       nshorts = (8 - (nshorts2 - nshorts));
       while(--nshorts >= 0) {
-        printf("     ");
+        log_write(LOG_PLAIN, "     ");
       }
-      if (!padding) printf("     ");
+      if (!padding) log_write(LOG_PLAIN, "     ");
     }
-    printf("  ");
+    log_write(LOG_PLAIN, "  ");
 
     while (--nshorts2 >= 0) {
-      printf("%c%c", asciify[*ap], asciify[*(ap+1)]);
+      log_write(LOG_PLAIN, "%c%c", asciify[*ap], asciify[*(ap+1)]);
       ap += 2;
       if ((++j % 8) == 0) {
-        printf("\n\t");
+        log_write(LOG_PLAIN, "\n\t");
         break;
       }
     }
     if (nshorts2 < 0) {
       if ((length & 1) && (((j-1) % 8) != 0)) {
-        printf("%c", asciify[*ap]);
+        log_write(LOG_PLAIN, "%c", asciify[*ap]);
       }
       break;
     }
   }
   if ((length & 1) && (((i-1) % 8) == 0)) {
-    printf(" %02x", *(u_char *)sp);
-    printf("                                       %c", asciify[*ap]);
+    log_write(LOG_PLAIN, " %02x", *(u_char *)sp);
+    log_write(LOG_PLAIN, "                                       %c", asciify[*ap]);
   }
-  printf("\n");
+  log_write(LOG_PLAIN, "\n");
 }
 
 #ifndef HAVE_STRERROR
@@ -194,75 +238,36 @@ char *strerror(int errnum) {
 }
 #endif
 
-/* Like the perl equivialent -- It removes the terminating newline from string
+/* Like the perl equivalent -- It removes the terminating newline from string
    IF one exists.  It then returns the POSSIBLY MODIFIED string */
 char *chomp(char *string) {
-  int len;
-  len = strlen(string);
-  if (len < 1)
-    return string;
-  if (string[len - 1] != '\n')
-    return string;  
-  if (len > 1 && string[len-2] == '\r') {
-    string[len-2] = '\0';
-  } else string[len-1] = '\0';
+  int len = strlen(string);
+  if (len && string[len - 1] == '\n') {
+    if (len > 1 && string[len - 2] == '\r')
+      string[len - 2] = '\0';
+    else
+      string[len - 1] = '\0';
+  }
   return string;
 }
 
-/* Convert a comma-separated list of ASCII u16-sized numbers into the
-   given 'dest' array, which is of total size (meaning sizeof() as
-   opposed to numelements) of destsize.  If min_elem and max_elem are
-   provided, each number must be within (or equal to) those
-   constraints.  The number of numbers stored in 'dest' is returned,
-   except that -1 is returned in the case of an error. If -1 is
-   returned and errorstr is non-null, *errorstr is filled with a ptr to a
-   static string literal describing the error. */
-
-int numberlist2array(char *expr, u16 *dest, int destsize, char **errorstr, u16 min_elem, u16 max_elem) {
-  char *current_range;
-  char *endptr;
-  char *errbogus;
-  long val;
-  int max_vals = destsize / 2;
-  int num_vals_saved = 0;
-  current_range = expr;
-
-  if (!errorstr)
-    errorstr = &errbogus;
-
-  if (destsize % 2 != 0) {
-    *errorstr = "Bogus call to numerlist2array() -- destsize must be a multiple of 2";
-    return -1;
+/* Compare a canonical option name (e.g. "max-scan-delay") with a
+   user-generated option such as "max_scan_delay" and returns 0 if the
+   two values are considered equivalant (for example, - and _ are
+   considered to be the same), nonzero otherwise. */
+int optcmp(const char *a, const char *b) {
+  while(*a && *b) {
+    if (*a == '_' || *a == '-') {
+      if (*b != '_' && *b != '-')
+	return 1;
+    }
+    else if (*a != *b)
+      return 1;
+    a++; b++;
   }
-
-  if (!expr || !*expr)
-    return 0;
-
-  do {
-    if (num_vals_saved == max_vals) {
-      *errorstr = "Buffer would overflow -- too many numbers in provided list";
-      return -1;
-    }
-    if( !isdigit((int) *current_range) ) {
-      *errorstr = "Alleged number begins with nondigit!  Example of proper form: \"20,80,65532\"";
-      return -1;
-    }
-    val = strtol(current_range, &endptr, 10);
-    if( val < min_elem || val > max_elem ) {
-      *errorstr = "Number given in list is outside given legal range";
-      return -1;
-    }
-    dest[num_vals_saved++] = (u16) val;
-    current_range = endptr;
-    while (*current_range == ',' || isspace(*current_range))
-      current_range++;
-    if (*current_range && !isdigit(*current_range)) {
-      *errorstr = "Bogus character in supposed number-list string. Example of proper form: \"20,80,65532\"";
-      return -1;
-    }
-  } while( current_range && *current_range);
-
-  return num_vals_saved;
+  if (*a || *b)
+    return 1;
+  return 0;
 }
 
 /* Scramble the contents of an array*/
@@ -277,7 +282,7 @@ unsigned char *tmp;
 int bpe;
 
 if (sizeof(unsigned char) != 1)
-  fatal("genfry() requires 1 byte chars");
+  fatal("%s() requires 1 byte chars", __func__);
 
 if (num_elem < 2)
   return;
@@ -294,8 +299,8 @@ else if (num_elem < 65536)
   bpe = sizeof(unsigned short);
 else bpe = sizeof(unsigned int);
 
-bytes = (unsigned char *) malloc(bpe * num_elem);
-tmp = (unsigned char *) malloc(elem_sz);
+bytes = (unsigned char *) safe_malloc(bpe * num_elem);
+tmp = (unsigned char *) safe_malloc(elem_sz);
 
 get_random_bytes(bytes, bpe * num_elem);
 cptr = bytes;
@@ -330,6 +335,7 @@ if (num_elem < 2)
  
  for(i= num_elem - 1; i > 0 ; i--) {
    num = get_random_ushort() % (i + 1);
+   if (i == num) continue;
    tmp = arr[i];
    arr[i] = arr[num];
    arr[num] = tmp;
@@ -353,71 +359,6 @@ int Send(int sd, const void *msg, size_t len, int flags) {
   return (res < 0)? -1 : (int) len;
 }
 
-// Write data to a file descriptor, keep retrying until an error or the full length
-// is written.  Returns -1 if there is an error, or len if the full length was sent.
-// Note that this does NOT work well on Windows using sockets -- so use Send() above 
-// for those.  I don't know if it works with regular files on Windows with files).
-ssize_t Write(int fd, const void *buf, size_t count) {
-  int res;
-  unsigned int len;
-
-  len = 0;
-  do {
-    res = write(fd,(char *) buf + len,count - len);
-    if (res > 0)
-      len += res;
-  } while(len < count && (res != -1 || socket_errno() == EINTR));
-
-  return (res == -1)? -1 : (int) count;
-}
-
-
-/* gcd_1 and gcd_n_long were sent in by Peter Kosinar <goober@gjh.sk> 
-   Not needed for gcd_n_long, just for the case you'd want to have gcd
-   for two arguments too. */
-unsigned long gcd_ulong(unsigned long a, unsigned long b)
-{
-  /* Shorter
-     while (b) { a%=b; if (!a) return b; b%=a; } */
-  
-  /* Faster */
-  unsigned long c; 
-  if (a<b) { c=a; a=b; b=c; }
-  while (b) { c=a%b; a=b; b=c; }
-  
-  /* Common for both */
-  return a;
-}
-
-unsigned long gcd_n_ulong(long nvals, unsigned long *val)
- {
-   unsigned long a,b,c;
-   
-   if (!nvals) return 1;
-   a=*val;
-   for (nvals--;nvals;nvals--)
-     {
-       b=*++val;
-       if (a<b) { c=a; a=b; b=c; }
-       while (b) { c=a%b; a=b; b=c; }
-     }
-   return a;
- }
-
-unsigned int gcd_uint(unsigned int a, unsigned int b)
-{
-  /* Shorter
-     while (b) { a%=b; if (!a) return b; b%=a; } */
-  
-  /* Faster */
-  unsigned int c; 
-  if (a<b) { c=a; a=b; b=c; }
-  while (b) { c=a%b; a=b; b=c; }
-  
-  /* Common for both */
-  return a;
-}
-
 unsigned int gcd_n_uint(int nvals, unsigned int *val)
  {
    unsigned int a,b,c;
@@ -434,11 +375,11 @@ unsigned int gcd_n_uint(int nvals, unsigned int *val)
  }
 
 /* This function takes a command and the address of an uninitialized
-   char ** .  It parses the command (by seperating out whitespace)
+   char ** .  It parses the command (by separating out whitespace)
    into an argv[] style char **, which it sets the argv parameter to.
    The function returns the number of items filled up in the array
    (argc), or -1 in the case of an error.  This function allocates
-   memmory for argv and thus it must be freed -- use argv_parse_free()
+   memory for argv and thus it must be freed -- use argv_parse_free()
    for that.  If arg_parse returns <1, then argv does not need to be freed.
    The returned arrays are always terminated with a NULL pointer */
 int arg_parse(const char *command, char ***argv) {
@@ -452,7 +393,7 @@ int arg_parse(const char *command, char ***argv) {
   if (Strncpy(mycommand, command, 4096) == -1) {      
     return -1;
   }
-  myargv = (char **) malloc((MAX_PARSE_ARGS + 2) * sizeof(char *));
+  myargv = (char **) safe_malloc((MAX_PARSE_ARGS + 2) * sizeof(char *));
   memset(myargv, 0, (MAX_PARSE_ARGS+2) * sizeof(char *));
   myargv[0] = (char *) 0x123456; /* Integrity checker */
   myargv++;
@@ -508,6 +449,28 @@ void arg_parse_free(char **argv) {
   free(argv);
 }
 
+/* Converts an Nmap time specification string into milliseconds.  If
+   the string is a plain non-negative number, it is considered to
+   already be in milliseconds and is returned.  If it is a number
+   followed by 's' (for seconds), 'm' (minutes), or 'h' (hours), the
+   number is converted to milliseconds and returned.  If Nmap cannot
+   parse the string, it is returned instead. */
+long tval2msecs(char *tspec) {
+  long l;
+  char *endptr = NULL;
+  l = strtol(tspec, &endptr, 10);
+  if (l < 0 || !endptr) return -1;
+  if (*endptr == '\0') return l;
+  if (*endptr == 's' || *endptr == 'S') return l * 1000;
+  if ((*endptr == 'm' || *endptr == 'M')) {
+    if (*(endptr + 1) == 's' || *(endptr + 1) == 'S') 
+      return l;
+    return l * 60000;
+  }
+  if (*endptr == 'h' || *endptr == 'H') return l * 3600000;
+  return -1;
+}
+
 // A simple function to form a character from 2 hex digits in ASCII form
 static unsigned char hex2char(unsigned char a, unsigned char b)
 {
@@ -584,6 +547,362 @@ char *cstring_unescape(char *str, unsigned int *newlen) {
   return str;
 }
 
+/* This function converts zero-terminated 'txt' string to binary 'data'.
+   It is used to parse user input for ip options. Some examples of possible input
+   strings and results:
+   	'\x01*2\xA2'	-> [0x01,0x01,0xA2]	// with 'x' number is parsed in hex
+   	'\01\01\255'	-> [0x01,0x01,0xFF]	// without 'x' its in decimal
+   	'\x01\x00*2'	-> [0x01,0x00,0x00]	// '*' is copying char
+   	'R'		-> Record Route with 9 slots
+   	'S 192.168.0.1 172.16.0.1' -> Strict Route with 2 slots
+   	'L 192.168.0.1 172.16.0.1' -> Loose Route with 2 slots
+   	'T'		-> Record Timestamp with 9 slots
+   	'U'		-> Record Timestamp and Ip Address with 4 slots
+*/
+int parse_ip_options(char *txt, u8 *data, int datalen, int* firsthopoff, int* lasthopoff){
+  enum{
+    NONE  = 0,
+    SLASH = 1,
+    MUL   = 2,
+    RR	  = 3,
+    TIME  = 4,
+  } s = NONE;
+  char *n, lc;
+  char *c = txt;
+  u8 *d = data;
+  int i,j;
+  int base = 10;
+  u8 *dataend = &data[datalen];
+  u8 *len = NULL;
+  char buf[32];
+  memset(data, 0, datalen);
+  bool sourcerouting = false;
+  
+
+  for(;*c;c++){
+    switch(s){
+    case SLASH:
+      // parse \x00 string
+      if(*c == 'x'){// just ignore this char
+      	base = 16;
+        break;
+      }
+      if(isxdigit(*c)){
+        *d++ = strtol(c, &n, base);
+        c=n-1;
+      }else
+        fatal("not a digit after '\\'");
+      s = NONE;
+      break;
+    case MUL:
+      if(d==data)
+        fatal("nothing before '*' char");
+      i = strtol(c, &n, 10);
+      if(i<2)
+        fatal("bad number after '*'");
+      c = n-1;		// move current txt pointer
+      lc = *(d-1);	// last char, we'll copy this
+      for(j=1; j<i; j++){
+        *d++ = lc;
+        if(d == dataend) // check for overflow
+          goto after;
+      }
+      s = NONE;
+      break;
+    case RR:
+      if(*c==' ' || *c==',')
+        break;
+      n = buf;
+      while((*c=='.' || (*c>='0' && *c<='9')) && n-buf <= ((int)sizeof(buf)-1))
+      	 *n++ = *c++;
+      *n = '\0'; c--;
+      if(d+4>=dataend)
+        fatal("Buffer too small. Or input data too big :)");
+      i = inet_pton(AF_INET, buf, d);
+      if(i<1)
+        fatal("Not a valid ipv4 address '%s'",buf);
+      // remember offset of first hop
+      if(sourcerouting && !*firsthopoff)
+        *firsthopoff = d - data;
+      d+=4;
+      if(*len<37)
+        *len += 4;
+      break;
+    case TIME:
+      fatal("No more arguments allowed!");
+    default:
+      switch(*c){
+      case '\\':s = SLASH;base=10;break;
+      case '*':s = MUL;break;
+      case 'R':
+      case 'S':
+      case 'L':
+        if(d != data)
+          fatal("This option can't be used in that way");
+        *d++ = '\x01';//NOP
+        switch(*c){
+        case 'R':*d++ = 7;break;
+        case 'S':*d++ = 137; sourcerouting=true; break;
+        case 'L':*d++ = 131; sourcerouting=true; break;
+        }
+	len = d;
+        *d++ = (*c=='R')? 39 : 3; // length: 3+4*9 bytes
+        *d++ = 4; //pointer          
+        s = RR;
+        break;
+      case 'T':
+      case 'U':
+        if(d != data)
+          fatal("This option can't be used in that way");
+	*d++ = 68;	// option type
+	len = d;
+        *d++ = (*c=='U') ? 36 : 40;   // length: 3+4*9 bytes or 4+4*9 bytes
+        *d++ = 5; // pointer
+        *d++ = (*c=='U') ? 1 : 0; // flag: address and Time fields
+        s = TIME;
+        break;
+      default://*d++ = *c;
+      	fatal("Bad character in ip option '%c'",*c);
+      }
+    }
+    if(d == dataend)
+      break;
+    assert(d<dataend);
+  }
+  if(sourcerouting){
+    if(*len<37){
+      *len+=4;
+      *lasthopoff = d - data;
+      *d++ = 0;*d++ = 0;*d++ = 0;*d++ = 0;
+    }else
+      fatal("When using source routing you must leave at least one slot for target's ip.");
+  }
+  if(s == RR)
+    return(*len+1); // because we inject NOP before
+  if(s == TIME)
+    return(*len);
+after:
+  return(d - data);
+}
+
+void bintohexstr(char *buf, int buflen, char *src, int srclen){
+    int bp=0;
+    int i;
+    for(i=0; i<srclen; i++){
+      bp += Snprintf(buf+bp, buflen-bp, "\\x%02hhx",src[i]);
+      if(bp >= buflen)break;
+      if(i%16==7){
+        bp += Snprintf(buf+bp, buflen-bp," ");
+        if(bp >= buflen)break;
+      }
+      if(i%16==15){
+        bp += Snprintf(buf+bp, buflen-bp,"\n");
+        if(bp >= buflen)break;
+      }
+    }
+    if(i%16!=0 && bp < buflen)
+      bp += Snprintf(buf+bp, buflen-bp,"\n");
+}
+
+static inline char* STRAPP(char *fmt, ...) {
+  static char buf[256];
+  static int bp;
+  int left = (int)sizeof(buf)-bp;
+  if(!fmt){
+    bp = 0;
+    return(buf);
+  }
+  if (left <= 0)
+    return buf;
+  va_list ap;
+  va_start(ap, fmt);
+  bp += Vsnprintf (buf+bp, left, fmt, ap);
+  va_end(ap);
+
+  return(buf);
+}
+
+#define HEXDUMP -2
+#define UNKNOWN -1
+
+#define BREAK()		\
+	{option_type = HEXDUMP; break;}
+#define CHECK(tt)	\
+  if(tt >= option_end)	\
+  	{option_type = HEXDUMP; break;}
+/* It tries to decode ip options.
+   Returns static buffer. watch out. */
+char* print_ip_options(u8* ipopt, int ipoptlen) {
+  char ipstring[32];
+  int option_type = UNKNOWN;// option type
+  int option_len  = 0; // option length
+  int option_pt   = 0; // option pointer
+  int option_fl   = 0;  // option flag
+  u8 *tptr;		// temp pointer
+  u32 *tint;		// temp int
+
+  int option_sta = 0;	// option start offset
+  int option_end = 0;	// option end offset
+  int pt = 0;		// current offset
+
+  // clear buffer
+  STRAPP(NULL,NULL);
+
+  if(!ipoptlen)
+    return(NULL);
+
+  while(pt<ipoptlen){	// for every char in ipopt
+    // read ip option header
+    if(option_type == UNKNOWN) {
+      option_sta  = pt;
+      option_type = ipopt[pt++];
+      if(option_type != 0 && option_type != 1) { // should we be interested in length field?
+        if(pt >= ipoptlen)	// no more chars
+          {option_type = HEXDUMP;pt--; option_end = 255; continue;} // no length field, hex dump to the end
+        option_len  = ipopt[pt++];
+        // end must not be greater than length
+        option_end  = MIN(option_sta + option_len, ipoptlen);
+        // end must not be smaller than current position
+        option_end  = MAX(option_end, option_sta+2);
+      }
+    }
+    switch(option_type) {
+    case 0:	// IPOPT_END
+    	STRAPP(" EOL", NULL);
+    	option_type = UNKNOWN;
+  	break;
+    case 1:	// IPOPT_NOP
+    	STRAPP(" NOP", NULL);
+    	option_type = UNKNOWN;
+  	break;
+/*    case 130:	// IPOPT_SECURITY
+    	option_type=-1;
+  	break;*/
+    case 131:	// IPOPT_LSRR	-> Loose Source and Record Route
+    case 137:	// IPOPT_SSRR	-> Strict Source and Record Route
+    case 7:	// IPOPT_RR	-> Record Route
+	if(pt - option_sta == 2) {
+    	  STRAPP(" %s%s{", (option_type==131)?"LS":(option_type==137)?"SS":"", "RR");
+    	  // option pointer
+    	  CHECK(pt);
+    	  option_pt = ipopt[pt++];
+    	  if(option_pt%4 != 0 || (option_sta + option_pt-1)>option_end || option_pt<4)	//bad or too big pointer
+    	    STRAPP(" [bad ptr=%02i]", option_pt);
+    	}
+    	if(pt - option_sta > 2) { // ip's
+    	  int i, s = (option_pt)%4;
+    	  // if pointer is mangled, fix it. it's max 3 bytes wrong
+    	  CHECK(pt+3);
+    	  for(i=0; i<s; i++)
+    	    STRAPP("\\x%02x", ipopt[pt++]);
+    	  option_pt -= i;
+    	  // okay, now we can start printing ip's
+    	  CHECK(pt+3);
+	  tptr = &ipopt[pt]; pt+=4;
+	  if(inet_ntop(AF_INET, (char *) tptr, ipstring, sizeof(ipstring)) == NULL)
+	    fatal("Failed to convert target address to presentation format!?!  Error: %s", strerror(socket_errno()));
+    	  STRAPP("%c%s",(pt-3-option_sta)==option_pt?'#':' ', ipstring);
+    	  if(pt == option_end)
+    	    STRAPP("%s",(pt-option_sta)==(option_pt-1)?"#":""); // pointer in the end?
+    	}else BREAK();
+  	break;
+    case 68:	// IPOPT_TS	-> Internet Timestamp
+	if(pt - option_sta == 2){
+	  STRAPP(" TM{");
+    	  // pointer
+    	  CHECK(pt);
+    	  option_pt  = ipopt[pt++];
+	  // bad or too big pointer
+    	  if(option_pt%4 != 1 || (option_sta + option_pt-1)>option_end || option_pt<5)
+    	    STRAPP(" [bad ptr=%02i]", option_pt);
+    	  // flags + overflow
+    	  CHECK(pt);
+    	  option_fl  = ipopt[pt++];
+    	  if((option_fl&0x0C) || (option_fl&0x03)==2)
+    	    STRAPP(" [bad flags=\\x%01hhx]", option_fl&0x0F);
+  	  STRAPP("[%i hosts not recorded]", option_fl>>4);
+  	  option_fl &= 0x03;
+	}
+    	if(pt - option_sta > 2) {// ip's
+    	  int i, s = (option_pt+3)%(option_fl==0?4:8);
+    	  // if pointer is mangled, fix it. it's max 3 bytes wrong
+    	  CHECK(pt+(option_fl==0?3:7));
+    	  for(i=0; i<s; i++)
+    	    STRAPP("\\x%02x", ipopt[pt++]);
+    	  option_pt-=i;
+
+	  // print pt
+  	  STRAPP("%c",(pt+1-option_sta)==option_pt?'#':' ');
+    	  // okay, first grab ip.
+    	  if(option_fl!=0){
+    	    CHECK(pt+3);
+	    tptr = &ipopt[pt]; pt+=4;
+	    if(inet_ntop(AF_INET, (char *) tptr, ipstring, sizeof(ipstring)) == NULL)
+	      fatal("Failed to convert target address to presentation format!?!  Error: %s", strerror(socket_errno()));
+	    STRAPP("%s@", ipstring);
+    	  }
+    	  CHECK(pt+3);
+	  tint = (u32*)&ipopt[pt]; pt+=4;
+	  STRAPP("%u", ntohl(*tint));
+
+    	  if(pt == option_end)
+  	    STRAPP("%s",(pt-option_sta)==(option_pt-1)?"#":" ");
+    	}else BREAK();
+  	break;
+    case 136:	// IPOPT_SATID	-> (SANET) Stream Identifier
+	if(pt - option_sta == 2){
+	  u16 *sh;
+    	  STRAPP(" SI{",NULL);
+    	  // length
+    	  if(option_sta+option_len > ipoptlen || option_len!=4)
+    	    STRAPP("[bad len %02i]", option_len);
+
+    	  // stream id
+    	  CHECK(pt+1);
+    	  sh = (u16*) &ipopt[pt]; pt+=2;
+    	  option_pt  = ntohs(*sh);
+    	  STRAPP("id=%i", option_pt);
+    	  if(pt != option_end)
+    	    BREAK();
+	}else BREAK();
+  	break;
+    case UNKNOWN:
+    default:
+    	// we read option_type and option_len, print them.
+    	STRAPP(" ??{\\x%02hhx\\x%02hhx", option_type, option_len);
+    	// check option_end once more:
+    	if(option_len < ipoptlen)
+    	  option_end = MIN(MAX(option_sta+option_len, option_sta+2),ipoptlen);
+    	else
+    	  option_end = 255;
+    	option_type = HEXDUMP;
+    	break;
+    case HEXDUMP:
+    	assert(pt<=option_end);
+    	if(pt == option_end){
+	  STRAPP("}",NULL);
+    	  option_type=-1;
+    	  break;
+    	}
+	STRAPP("\\x%02hhx", ipopt[pt++]);
+    	break;
+    }
+    if(pt == option_end && option_type != UNKNOWN) {
+      STRAPP("}",NULL);
+      option_type = UNKNOWN;
+    }
+  } // while 
+  if(option_type != UNKNOWN)
+    STRAPP("}");
+
+  return(STRAPP("",NULL));
+}
+#undef CHECK
+#undef BREAK
+#undef UNKNOWN
+#undef HEXDUMP
+
+
 /* mmap() an entire file into the address space.  Returns a pointer
    to the beginning of the file.  The mmap'ed length is returned
    inside the length parameter.  If there is a problem, NULL is
@@ -655,7 +974,7 @@ char *mmapfile(char *fname, int *length, int openflags)
   }
  else {
   oflags = GENERIC_READ | GENERIC_WRITE;
-  mflags = PAGE_READONLY | PAGE_READWRITE;
+  mflags = PAGE_READWRITE;
  }
 
  fd = CreateFile (
@@ -684,8 +1003,8 @@ char *mmapfile(char *fname, int *length, int openflags)
  CloseHandle (fd);
 
  if (o.debugging > 2)
-  printf ("mmapfile(): fd %08lX, gmap %08lX, fileptr %08lX, length %d\n",
-    (DWORD)fd, (DWORD)gmap, (DWORD)fileptr, *length);
+  log_write(LOG_PLAIN, "%s(): fd %08lX, gmap %08lX, fileptr %08lX, length %d\n",
+    __func__, (DWORD)fd, (DWORD)gmap, (DWORD)fileptr, *length);
 
 	return fileptr;
 }
@@ -696,7 +1015,7 @@ char *mmapfile(char *fname, int *length, int openflags)
 int win32_munmap(char *filestr, int filelen)
 {
   if (gmap == 0)
-    fatal("win32_munmap: no current mapping !\n");
+    fatal("%s: no current mapping !\n", __func__);
 
   FlushViewOfFile(filestr, filelen);
   UnmapViewOfFile(filestr);
